﻿/*
plugapi.cpp

API, доступное плагинам (диалоги, меню, ...)
*/
/*
Copyright © 1996 Eugene Roshal
Copyright © 2000 Far Group
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. The name of the authors may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "headers.hpp"
#pragma hdrstop

#include "plugapi.hpp"
#include "keys.hpp"
#include "help.hpp"
#include "vmenu.hpp"
#include "vmenu2.hpp"
#include "dialog.hpp"
#include "filepanels.hpp"
#include "panel.hpp"
#include "cmdline.hpp"
#include "scantree.hpp"
#include "fileview.hpp"
#include "fileedit.hpp"
#include "plugins.hpp"
#include "savescr.hpp"
#include "flink.hpp"
#include "manager.hpp"
#include "ctrlobj.hpp"
#include "window.hpp"
#include "scrbuf.hpp"
#include "TPreRedrawFunc.hpp"
#include "syslog.hpp"
#include "interf.hpp"
#include "keyboard.hpp"
#include "colormix.hpp"
#include "message.hpp"
#include "eject.hpp"
#include "filefilter.hpp"
#include "fileowner.hpp"
#include "stddlg.hpp"
#include "pathmix.hpp"
#include "exitcode.hpp"
#include "processname.hpp"
#include "RegExp.hpp"
#include "TaskBar.hpp"
#include "console.hpp"
#include "plugsettings.hpp"
#include "farversion.hpp"
#include "mix.hpp"
#include "FarGuid.hpp"
#include "clipboard.hpp"
#include "strmix.hpp"
#include "notification.hpp"
#include "panelmix.hpp"
#include "xlat.hpp"
#include "dirinfo.hpp"
#include "lang.hpp"
#include "viewer.hpp"
#include "datetime.hpp"
#include "string_utils.hpp"
#include "cvtname.hpp"
#include "filemasks.hpp"
#include "desktop.hpp"

int WINAPIV apiSprintfA(char* Dest, const char* Format, ...) noexcept //?deprecated
{
	// noexcept
	va_list argptr;
	va_start(argptr, Format);
 	int ret = vsprintf(Dest, Format, argptr);
	va_end(argptr);
	return ret;
}

int WINAPIV apiSnprintfA(char* Dest, size_t Count, const char* Format, ...) noexcept
{
	// noexcept
	va_list argptr;
	va_start(argptr, Format);
	int ret = _vsnprintf(Dest, Count, Format, argptr);
	va_end(argptr);
	return ret;
}

int WINAPIV apiSscanfA(const char* Src, const char* Format, ...) noexcept
{
	va_list argptr;
	va_start(argptr, Format);
	int ret = vsscanf(Src, Format, argptr);
	va_end(argptr);
	return ret;
}

int WINAPIV apiSprintfW(wchar_t* Dest, const wchar_t* Format, ...) noexcept //?deprecated
{
	// noexcept
	va_list argptr;
	va_start(argptr, Format);
	// BUGBUG, do not use vswprintf here, %s treated as char* in GCC
	int ret = _vsnwprintf(Dest, 32000, Format, argptr);
	va_end(argptr);
	return ret;
}

int WINAPIV apiSnprintfW(wchar_t* Dest, size_t Count, const wchar_t* Format, ...) noexcept
{
	// noexcept
	va_list argptr;
	va_start(argptr, Format);
	// BUGBUG, do not use vswprintf here, %s treated as char* in GCC
	int ret = _vsnwprintf(Dest, Count, Format, argptr);
	va_end(argptr);
	return ret;
}

int WINAPIV apiSscanfW(const wchar_t* Src, const wchar_t* Format, ...) noexcept
{
	va_list argptr;
	va_start(argptr, Format);
	int ret = vswscanf(Src, Format, argptr);
	va_end(argptr);
	return ret;
}

size_t WINAPI apiConvertPath(
	CONVERTPATHMODES Mode, const wchar_t *Src,
	const wchar_t *CurrentDirectory,
	wchar_t *Dest, size_t DestSize) noexcept
{
	string orig_current_directory;
	if (CurrentDirectory)
	{
		orig_current_directory = os::fs::GetCurrentDirectoryW();
		os::fs::SetCurrentDirectoryW(CurrentDirectory, false);
	}

	string strDest;

	switch (Mode)
	{
	case CPM_NATIVE:
		strDest=NTPath(Src);
		break;

	case CPM_REAL:
		strDest = ConvertNameToReal(Src);
		break;

	case CPM_FULL:
	default:
		strDest = ConvertNameToFull(Src);
		break;
	}

	if (Dest && DestSize)
		xwcsncpy(Dest, strDest.data(), DestSize);

	if (CurrentDirectory)
	{
		os::fs::SetCurrentDirectoryW(orig_current_directory, false);
	}
	return strDest.size() + 1;
}

size_t WINAPI apiGetPathRoot(const wchar_t *Path, wchar_t *Root, size_t DestSize) noexcept
{
	const auto strRoot = GetPathRoot(Path);

	if (DestSize && Root)
		xwcsncpy(Root,strRoot.data(),DestSize);

	return strRoot.size()+1;
}

intptr_t WINAPI apiRegExpControl(HANDLE hHandle, FAR_REGEXP_CONTROL_COMMANDS Command, intptr_t Param1, void* Param2) noexcept
{
	RegExp* re = nullptr;

	if (Command != RECTL_CREATE)
	{
		if (!hHandle || hHandle == INVALID_HANDLE_VALUE)
			return FALSE;

		re = static_cast<RegExp*>(hHandle);
	}

	switch (Command)
	{
	case RECTL_CREATE:
		*static_cast<HANDLE*>(Param2) = INVALID_HANDLE_VALUE;
		re = new RegExp;

		*static_cast<HANDLE*>(Param2) = static_cast<HANDLE>(re);
		return TRUE;

	case RECTL_FREE:
		delete re;
		return TRUE;

	case RECTL_COMPILE:
		return re->Compile(static_cast<const wchar_t*>(Param2), OP_PERLSTYLE);

	case RECTL_OPTIMIZE:
		return re->Optimize();

	case RECTL_MATCHEX:
	{
		const auto data = static_cast<RegExpSearch*>(Param2);
		return re->MatchEx(data->Text, data->Text + data->Position, data->Text + data->Length, data->Match, data->Count);
	}

	case RECTL_SEARCHEX:
	{
		const auto data = static_cast<RegExpSearch*>(Param2);
		return re->SearchEx(data->Text, data->Text + data->Position, data->Text + data->Length, data->Match, data->Count);
	}

	case RECTL_BRACKETSCOUNT:
		return re->GetBracketsCount();

	default:
		return FALSE;
	}
}

static void init_keys()
{
	static bool done = false;
	if (!done)
	{
		done = true;
		InitKeysArray();
	}
}

size_t WINAPI apiInputRecordToKeyName(const INPUT_RECORD* Key, wchar_t *KeyText, size_t Size) noexcept
{
	init_keys();
	int iKey = InputRecordToKey(Key);
	string strKT;
	if (iKey == KEY_NONE || !KeyToText(iKey, strKT))
		return 0;
	size_t len = strKT.size();
	if (Size && KeyText)
	{
		if (Size <= len)
			len = Size - 1;
		std::copy_n(strKT.data(), len, KeyText);
		KeyText[len] = 0;
	}
	else if (KeyText)
		*KeyText = 0;
	return len + 1;
}

BOOL WINAPI apiKeyNameToInputRecord(const wchar_t *Name, INPUT_RECORD* RecKey) noexcept
{
	init_keys();
	int Key = KeyNameToKey(Name);
	return Key > 0 ? KeyToInputRecord(Key, RecKey) : FALSE;
}
